package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.dsmongo.api.constant.enums.MongoOperTypeEnum;
import com.gic.dsmongo.api.dto.Criteria;
import com.gic.dsmongo.api.dto.MongoQueryDTO;
import com.gic.dsmongo.api.dto.MongoSaveDTO;
import com.gic.dsmongo.api.dto.MongoUpdateDTO;
import com.gic.dsmongo.api.service.MongoOperApiService;
import com.gic.enterprise.entity.CustomSettingData;
import com.gic.enterprise.entity.TabTableSetting;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.CustomSettingApiService;
import com.gic.enterprise.service.TableSettingFieldService;
import com.gic.enterprise.service.TableSettingService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service("customSettingApiService")
public class CustomSettingApiServiceImpl implements CustomSettingApiService {
    private static final Logger log = LogManager.getLogger(CustomSettingApiServiceImpl.class);
    private static final String DATABASENAME = "custom_table";
    private static final String PRIMARYKEY = "_id";
    @Autowired
    private MongoOperApiService mongoOperApiService;
    @Autowired
    TableSettingService tableSettingService;
    @Autowired
    TableSettingFieldService tableSettingFieldService;

    @Override
    public ServiceResponse<String> save(String tableName, String params) {
        log.info("保存参数：{}，{}", tableName, params);
        TabTableSetting tableSetting = tableSettingService.getTableSetting(tableName);
        if(tableSetting == null){
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), tableName + "不存在");
        }
        Map<String, Object> map = JSON.parseObject(params, Map.class);
        String repeateSetting = tableSetting.getRepeateSetting();
        if(StringUtils.isNotBlank(repeateSetting)){
            String[] split = repeateSetting.split(",");
            List<Criteria> criteriaList = new ArrayList<>();
            for(String field : split){
                Criteria criteria = new Criteria(field, MongoOperTypeEnum.EQ, map.get(field));
                criteriaList.add(criteria);
            }
            if(map.get("_id") != null){
                Criteria criteria = new Criteria(PRIMARYKEY, MongoOperTypeEnum.NIN, map.get(PRIMARYKEY));
                criteriaList.add(criteria);
            }
            MongoQueryDTO queryDTO = new MongoQueryDTO();
            queryDTO.setCriteria(criteriaList);
            queryDTO.setDataBaseName(DATABASENAME);
            queryDTO.setConllectionName(tableName);
            ServiceResponse<List<Map>> query = mongoOperApiService.query(queryDTO);
            if(CollectionUtils.isNotEmpty(query.getResult())){
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "存在重复记录");
            }
        }
        if(map.get(PRIMARYKEY) != null){
            //更新
            MongoUpdateDTO mongoUpdateDTO = new MongoUpdateDTO();
            mongoUpdateDTO.setDataBaseAndCollectionName(DATABASENAME, tableName);
            mongoUpdateDTO.setData(map);
            ServiceResponse<Integer> update = mongoOperApiService.update(mongoUpdateDTO);
            log.info("update:{}", JSON.toJSONString(update));
        } else {
            MongoSaveDTO mongoSaveDTO = new MongoSaveDTO();
            mongoSaveDTO.setDataBaseAndCollectionName(DATABASENAME, tableName);
            mongoSaveDTO.setData(map);
            ServiceResponse<String> save = mongoOperApiService.save(mongoSaveDTO);
            log.info("save:{}", JSON.toJSONString(save));
        }
        return ServiceResponse.success();
    }
}
