package com.gic.enterprise.web.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.web.vo.ShelfCouponVO;
import com.gic.marketing.api.service.coupon.CouponShelfApiService;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constant.applet.*;
import com.gic.enterprise.qo.AppletPageQO;
import com.gic.enterprise.service.AppletCustomPageApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.AppletPageDetailVO;
import com.gic.enterprise.web.vo.AppletPageVO;
/**
 *
 * @ClassName: CustomPageController
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:28
 */
@RestController
@RequestMapping("/custom-page")
public class CustomPageController {

    private static final Logger LOGGER = LogManager.getLogger(CustomPageController.class);

    @Autowired
    private AppletCustomPageApiService appletCustomPageApiService;
    @Autowired
    private CouponShelfApiService couponShelfApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;

    @RequestMapping("/save-page")
    public RestResponse savePage(String param, Integer pageId, Integer saveType, Integer appType, String appId) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(
                appletCustomPageApiService.saveOrUpdate(param, pageId, enterpriseId, saveType, appType, appId));
    }

    @RequestMapping("/list-page")
    public RestResponse listPage(AppletPageQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        params.setOrderByStr(AppletPageOrderByEnum.getOrderBySqlByCode(params.getOrderByCode()));
        return ResultControllerUtils.commonPageResult(appletCustomPageApiService.page(params), AppletPageVO.class);
    }

    @RequestMapping("/get-detail")
    public RestResponse getDetail(Integer pageId) {
        return ResultControllerUtils.commonResultOne(appletCustomPageApiService.getDetailByPageId(pageId),
                AppletPageDetailVO.class);
    }

    @RequestMapping("/enable")
    public RestResponse enable(Integer pageId) {
        ServiceResponse<String> result = appletCustomPageApiService.enable(pageId);
        return OperationResultUtils.operationResult(result, "发布装修页面", OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/disable")
    public RestResponse disable(Integer pageId) {
        ServiceResponse<String> result = appletCustomPageApiService.disable(pageId);
        return OperationResultUtils.operationResult(result, "关闭装修页面", OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/set-serial")
    public RestResponse setSerial(Integer pageId, Integer serialNumber) {
        ServiceResponse<Void> result = appletCustomPageApiService.setSerialNumber(pageId, serialNumber);
        return OperationResultUtils.operationResult(result, "设置页面序号");
    }

    @RequestMapping("/delete")
    public RestResponse delete(Integer pageId) {
        ServiceResponse<String> result = appletCustomPageApiService.delete(pageId);
        return OperationResultUtils.operationResult(result, "删除装修页面", OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/delete-fetch")
    public RestResponse deleteFetch(String pageIds) {
        ServiceResponse<String> result = appletCustomPageApiService.deleteFetch(pageIds);
        return OperationResultUtils.operationResult(result, "删除装修页面", OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/list-shelf-coupon")
    public RestResponse listShelfCoupon(String memberCardId, String appId) {
        if (StringUtils.isBlank(appId)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数不能为空");
        }
        ServiceResponse<List<AppletsConfigSimpleOutputDTO>> result = appletsConfigApiService
                .listAppletConfigAppIdContainDel(UserDetailUtils.getUserDetail().getEnterpriseId());
        Long appletAreaId = 0L;
        if (result.isSuccess()) {
            List<AppletsConfigSimpleOutputDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                for (AppletsConfigSimpleOutputDTO dto : list) {
                    if (appId.equals(dto.getAppId())) {
                        appletAreaId = dto.getAppletConfigId();
                        break;
                    }
                }
            }
        }
        return ResultControllerUtils.commonResult(couponShelfApiService
                .listShelf(UserDetailUtils.getUserDetail().getEnterpriseId(), appletAreaId), ShelfCouponVO.class);
    }

    /**
     * 数据资产类型
     * @Title: listDataIcon 
     * @Description:
      * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-data-icon")
    public RestResponse listDataIcon() {
        List<Map<String, String>> list = new ArrayList<>();
        Map<String, String> result;
        for (AppletDataIconEnum type : AppletDataIconEnum.values()) {
            result = new HashMap<>(4);
            result.put("key", type.getCode());
            result.put("value", type.getMessage());
            list.add(result);
        }
        return RestResponse.success(list);
    }

    @RequestMapping("/list-component")
    public RestResponse listComponent() {
        List<Map<String, String>> list = new ArrayList<>();
        Map<String, String> result;
        for (AppletPageComponentEnum type : AppletPageComponentEnum.values()) {
            result = new HashMap<>(4);
            result.put("key", type.getCode());
            result.put("value", type.getMessage());
            list.add(result);
        }
        return RestResponse.success(list);
    }

    @RequestMapping("/list-page-type")
    public RestResponse listPageType() {
        List<Map<String, Object>> list = new ArrayList<>();
        Map<String, Object> result;
        for (AppletPageTypeEnum type : AppletPageTypeEnum.values()) {
            result = new HashMap<>(4);
            result.put("key", type.getCode());
            result.put("value", type.getMessage());
            list.add(result);
        }
        return RestResponse.success(list);
    }

    @RequestMapping("/list-entry-condition")
    public RestResponse listEntryCondition() {
        List<Map<String, Object>> list = new ArrayList<>();
        Map<String, Object> result;
        for (EntryConditionEnum type : EntryConditionEnum.values()) {
            result = new HashMap<>(4);
            result.put("key", type.getCode());
            result.put("value", type.getMessage());
            list.add(result);
        }
        return RestResponse.success(list);
    }
}
