package com.gic.enterprise.web.controller;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import com.gic.data.api.dto.index.IndexMemberOverviewDTO;
import com.gic.data.api.qo.index.IndexMemberOverviewQO;
import com.gic.enterprise.web.vo.index.MemberCardInfoVO;
import com.gic.member.config.api.dto.*;
import com.gic.member.config.api.service.UserListFieldApiService;
import com.gic.platform.member.constant.enums.CuTypeEnum;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.data.api.dto.*;
import com.gic.data.api.service.MemberDataApiService;
import com.gic.data.api.service.StoreStatisicsApiService;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.HelpCenterDTO;
import com.gic.enterprise.dto.IndexModuleDTO;
import com.gic.enterprise.dto.PushTypeDTO;
import com.gic.enterprise.dto.UserMessageDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.PageQO;
import com.gic.enterprise.service.HelpCenterApiService;
import com.gic.enterprise.service.IndexModuleApiService;
import com.gic.enterprise.service.PushMessageApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.MemberDataRightVO;
import com.gic.enterprise.web.vo.MemberDataVO;
import com.gic.enterprise.web.vo.ResouceVO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import com.gic.store.dto.CityDTO;
import com.gic.store.dto.ProvinceDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.service.ProvincesApiService;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreWidgetApiService;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-14 9:11
 */
@RestController
public class IndexModuleController {
    private static final Logger log = LogManager.getLogger(IndexModuleController.class);
    @Autowired
    private IndexModuleApiService indexModuleApiService;
    @Autowired
    private HelpCenterApiService helpCenterApiService;
    @Autowired
    private PushMessageApiService pushMessageApiService;
    @Autowired
    private StoreStatisicsApiService storeStatisicsApiService;
    @Autowired
    private ProvincesApiService provincesApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private MemberDataApiService memberDataApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;
    @Autowired
    private UserListFieldApiService userListFieldApiService;

    @RequestMapping("/list-index-module")
    public RestResponse listIndexModule() {
        ServiceResponse<List<IndexModuleDTO>> serviceResponse = indexModuleApiService.listIndexModuleEnterprise(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    /**
     * 首页会员卡列表数据，包括共享
     * @return
     */
    @RequestMapping("/list-member-card-of-index")
    public RestResponse listMemberCardOfIndex() {
        return RestResponse.success(getMbrArea(UserDetailUtils.getUserDetail().getUserId(), UserDetailUtils.getUserDetail().getEnterpriseId()));
    }

    @RequestMapping("list-help-center")
    public RestResponse listHelpCenter(){
        ServiceResponse<List<HelpCenterDTO>> response = this.helpCenterApiService.listHelpCenter();
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("my-message")
    public RestResponse myMessage(String search, Integer status, String startTime, String endTime, Integer pushClassifyId, PageQO pageQO){
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            ServiceResponse<Page<UserMessageDTO>> response = this.pushMessageApiService.pageUserMessage(userId, search, status, StringUtils.isBlank(startTime) ? null : sdf.parse(startTime+" 00:00:00"),
                    StringUtils.isBlank(endTime) ? null : sdf.parse(endTime+" 23:59:59"), pushClassifyId, pageQO.getCurrentPage(), pageQO.getPageSize());
            ServiceResponse<Integer> unReadResponse = this.pushMessageApiService.totalUserMessage(userId, search, 1, StringUtils.isBlank(startTime) ? null : sdf.parse(startTime+" 00:00:00"),
                    StringUtils.isBlank(endTime) ? null : sdf.parse(endTime+" 23:59:59"), pushClassifyId);
            Map<String, Object> result = new HashMap<>();
            result.put("page", response.getResult());
            result.put("unReadCount", unReadResponse.getResult());
            return RestResponse.success(result);
        } catch (ParseException e) {
            log.warn("异常", e);
            return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "日期转换错误");
        }
    }

    @RequestMapping("total-message")
    public RestResponse totalMessage(){
        ServiceResponse<Integer> response = this.pushMessageApiService.totalUserMessage(UserDetailUtils.getUserDetail().getUserId(), null, null, null,
                null, null);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("read-all")
    public RestResponse readAll(){
        ServiceResponse<Integer> response = this.pushMessageApiService.readAll(UserDetailUtils.getUserDetail().getUserId());
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("message-detail")
    public RestResponse messageDetail(Integer messageId){
        ServiceResponse<UserMessageDTO> userMessageDetail = this.pushMessageApiService.getUserMessageDetail(messageId);
        this.pushMessageApiService.readMessage(messageId);
        return RestResponse.success(userMessageDetail.getResult());
    }

    @RequestMapping("message-classify")
    public RestResponse listMessageClassify(){
        ServiceResponse<Page<PushTypeDTO>> response = this.pushMessageApiService.pagePushType(null, 1, 10000);
        return RestResponse.success(response.getResult().getResult());
    }

    @RequestMapping("store-data")
    public RestResponse storeData(Long regionId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<ProvinceDTO>> serviceResponse = this.provincesApiService.selectAllProvince();
        List<Integer> authStoreIdList = new ArrayList<>();
        if(UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() != 1){
            ServiceResponse<List<Integer>> authResponse = this.storeWidgetApiService.listStoreInfoIdByStoreWidgetId(enterpriseId, UserDetailUtils.getUserDetail().getStoreResourceId().intValue());
            if(authResponse.isSuccess()){
                authStoreIdList = authResponse.getResult();
            }
            if(CollectionUtils.isEmpty(authStoreIdList)){
                authStoreIdList.add(-1);
            }
        }

        ServiceResponse<List<StoreProvinceData>> dataResponse = this.storeStatisicsApiService.storeProvinceData(enterpriseId, regionId, authStoreIdList);
        ServiceResponse<List<StoreProvinceData>> dataOpenStore = this.storeStatisicsApiService.storeProvinceDataOpenStore(enterpriseId, regionId, authStoreIdList);
        List<Map<String, Object>> result = new ArrayList<>();
        if(serviceResponse.isSuccess() && serviceResponse.getResult() != null){
            for(ProvinceDTO provinceDTO : serviceResponse.getResult()){
                Map<String, Object> map = new HashMap<>();
                map.put("key", provinceDTO.getProvinceName());
                map.put("mainStoreMemberCount", 0);
                map.put("openStoreMemberCount", 0);
                if (dataResponse != null) {
                    List<StoreProvinceData> storeProvinceDataList = dataResponse.getResult();
                    if (CollectionUtils.isNotEmpty(storeProvinceDataList)) {
                        storeProvinceDataList.forEach(e -> {
                            if (provinceDTO.getProvinceName().equals(e.getProvince())) {
                                map.put("mainStoreMemberCount", e.getMemberCount());
                            }
                        });
                    }
                }
                if (dataOpenStore != null) {
                    List<StoreProvinceData> dataOpenStoreList = dataOpenStore.getResult();
                    if (CollectionUtils.isNotEmpty(dataOpenStoreList)) {
                        for (StoreProvinceData storeProvinceData : dataOpenStoreList) {
                            if (provinceDTO.getProvinceName().equals(storeProvinceData.getProvince())) {
                                map.put("openStoreMemberCount", storeProvinceData.getMemberCount());
                            }
                        }
                    }
                }
                //查询门店数量
                map.put("storeCount", this.getStoreNum(enterpriseId, null, provinceDTO.getProvinceId()));
                map.put("onlineStoreCount", this.getStoreNum(enterpriseId, 1, provinceDTO.getProvinceId()));
                result.add(map);
            }
        }
        Map<String, Object> data = new HashMap<>();
        data.put("data", result);
        data.put("legend", this.getLegend(enterpriseId, null));
        return RestResponse.success(data);
    }

    @RequestMapping("store-city-data")
    public RestResponse storeCityData(String province, Long regionId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<CityDTO>> serviceResponse = this.provincesApiService.selectAllCity();
        ServiceResponse<List<ProvinceDTO>> provinceResponse = this.provincesApiService.selectAllProvince();
        String provinceId = null;
        for(ProvinceDTO provinceDTO : provinceResponse.getResult()){
            if(provinceDTO.getProvinceName().equals(province)){
                provinceId = provinceDTO.getProvinceId();
            }
        }
        if(StringUtils.isBlank(provinceId)){
            return RestResponse.success();
        }
        List<Integer> authStoreIdList = new ArrayList<>();
        if(UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() != 1){
            ServiceResponse<List<Integer>> authResponse = this.storeWidgetApiService.listStoreInfoIdByStoreWidgetId(enterpriseId, UserDetailUtils.getUserDetail().getStoreResourceId().intValue());
            if(authResponse.isSuccess()){
                authStoreIdList = authResponse.getResult();
            }
            if(CollectionUtils.isEmpty(authStoreIdList)){
                authStoreIdList.add(-1);
            }
        }

        ServiceResponse<List<StoreCityData>> dataResponse = this.storeStatisicsApiService.storeCityData(enterpriseId, regionId, authStoreIdList, province);
        ServiceResponse<List<StoreCityData>> dataOpenStore = this.storeStatisicsApiService.storeCityDataOpenStore(enterpriseId, regionId, authStoreIdList, province);
        List<Map<String, Object>> result = new ArrayList<>();
        if(serviceResponse.isSuccess() && serviceResponse.getResult() != null){
            for(CityDTO cityDTO : serviceResponse.getResult()){
                if(cityDTO.getProvinceId().equals(provinceId)){
                    Map<String, Object> map = new HashMap<>();
                    map.put("key", cityDTO.getCityName());
                    map.put("mainStoreMemberCount", 0);
                    map.put("openStoreMemberCount", 0);
                    for(StoreCityData storeCityData : dataResponse.getResult()){
                        if(cityDTO.getCityName().equals(storeCityData.getCity())){
                            map.put("mainStoreMemberCount", storeCityData.getMemberCount());
                        }
                    }
                    for(StoreCityData storeCityData : dataOpenStore.getResult()){
                        if(cityDTO.getCityName().equals(storeCityData.getCity())){
                            map.put("openStoreMemberCount", storeCityData.getMemberCount());
                        }
                    }
                    //查询门店数量
                    map.put("storeCount", this.getStoreNum(enterpriseId, null, provinceId, cityDTO.getCityId()));
                    map.put("onlineStoreCount", this.getStoreNum(enterpriseId, 1, provinceId, cityDTO.getCityId()));
                    result.add(map);
                }
            }
        }
        Map<String, Object> data = new HashMap<>();
        data.put("data", result);
        data.put("legend", this.getLegend(enterpriseId, provinceId));
        return RestResponse.success(data);
    }


    @RequestMapping("list-member-data")
    public RestResponse listMember(Integer type, Long cardAreaId, Integer brandId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        IndexMemberOverviewQO param = new IndexMemberOverviewQO();
        param.setEntId(enterpriseId);
        param.setAuthStoreIdList(this.getAuthStore());
        param.setType(type);
        param.setBrandId(brandId);
        if (cardAreaId != null) {
            param.setMemberCardIdList(Arrays.asList(cardAreaId));
        } else {
            List<MemberCardInfoVO> memberCardList = getMbrArea(userId, enterpriseId);
            if (CollectionUtils.isNotEmpty(memberCardList)) {
                param.setMemberCardIdList(memberCardList.stream().mapToLong(e -> Long.valueOf(e.getMbrAreaId())).boxed().collect(Collectors.toList()));
            }
        }
        ServiceResponse<List<IndexMemberOverviewDTO>> response = this.memberDataApiService.listMemberData(param);
        return RestResponse.success(EntityUtil.changeEntityListByJSON(MemberDataVO.class, response.getResult()));
    }

    @RequestMapping("get-member-data")
    public RestResponse getMemberData(Integer type){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        ServiceResponse<MemberDataDTO> memberData = this.memberDataApiService.getMemberData(enterpriseId, type, userId, this.getAuthStore());
        return RestResponse.success(EntityUtil.changeEntityByJSON(MemberDataRightVO.class, memberData.getResult()));
    }

    @RequestMapping("list-member-channel-data")
    public RestResponse listMemberChannelData(Integer type, Long cardConfigId, Long appletConfigId, Long serviceConfigId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        ServiceResponse<List<MemberChannelDataDTO>> response = this.memberDataApiService.listMemberChannelData(type,cardConfigId, appletConfigId, serviceConfigId, enterpriseId, userId, this.getAuthStore());
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("get-member-channel-data")
    public RestResponse getMemberChannelData(Integer type){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<AdsGicMbrVennAllDTO> memberChannelData = this.memberDataApiService.getMemberChannelData(enterpriseId);
        return RestResponse.success(memberChannelData.getResult());
    }

    @RequestMapping("list-channel-data")
    public RestResponse listChannelData(Integer type, Integer cuType){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        ServiceResponse<List<MemberChannelDataDTO>> listServiceResponse = this.memberDataApiService.listChannelData(type, cuType, enterpriseId, userId, this.getAuthStore());
        return RestResponse.success(listServiceResponse.getResult());
    }


    @RequestMapping("consumption-analysis")
    public RestResponse consumptionAnalysis(Integer consumptionType, Long cardConfigId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        ServiceResponse<List<ConsumptionAnalysisDTO>> listServiceResponse = this.memberDataApiService.consumptionAnalysis(consumptionType, cardConfigId, enterpriseId, userId, this.getAuthStore());
        return RestResponse.success(listServiceResponse.getResult());
    }

    @RequestMapping("list-resource")
    public RestResponse listResource(){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        List<OwnerMemberCardDTO> memberCardList = memberCardApiService.getAll(enterpriseId).getResult();
        List<AppletsConfigSimpleOutputDTO> appletList = appletsConfigApiService.listValidAppletConfigAppId(enterpriseId).getResult();
        List<ServiceConfigSimpleOutputDTO> serviceList = serviceConfigApiService.listValidServiceConfigAppId(enterpriseId).getResult();
        Map<String, Object> result = new HashMap<>();
        List<ResouceVO> applet = new ArrayList<>();
        List<ResouceVO> service = new ArrayList<>();
        List<ResouceVO> card = new ArrayList<>();
        if( CollectionUtils.isNotEmpty(appletList)){
            for(AppletsConfigSimpleOutputDTO dto : appletList){
                ResouceVO vo = new ResouceVO();
                vo.setId(dto.getAppletConfigId());
                vo.setName(dto.getAppletConfigName());
                applet.add(vo);
            }
        }
        if(CollectionUtils.isNotEmpty(serviceList)){
            for(ServiceConfigSimpleOutputDTO dto : serviceList){
                ResouceVO vo = new ResouceVO();
                vo.setId(dto.getServiceConfigId());
                vo.setName(dto.getServiceConfigName());
                service.add(vo);
            }
        }
        if(CollectionUtils.isNotEmpty(memberCardList)){
            for(OwnerMemberCardDTO dto : memberCardList){
                ResouceVO vo = new ResouceVO();
                vo.setId(dto.getCardConfigId());
                vo.setName(dto.getCardName());
                card.add(vo);
            }
        }
        result.put("card", card);
        result.put("service", service);
        result.put("applet", applet);
        result.put("isSingleArea", card.size()<=1 && service.size()<=1 && applet.size()<=1);
        return RestResponse.success(result);
    }

    private List<Integer> getLegend(Integer enterpriseId, String provinceId){
        Integer storeNum = this.getStoreNum(enterpriseId, null, provinceId);
        List<Integer> legend = new ArrayList<>();
        Integer max = 15;
        if(storeNum <=max){
            int index = 1;
            while (storeNum>0){
                legend.add(index*1);
                storeNum -= index*1;
                index++;
            }
        }else {
            int sig = storeNum%15 == 0 ? storeNum/15 : storeNum/15+1;
            for(int i = 1; i<= Constants.FIVE; i++){
                legend.add(i*sig);
            }
        }
        return legend;
    }

    private Integer getStoreNum(Integer enterpriseId, Integer status, String provinceId){
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setStatus(status);
        storeSearchDTO.setProvinceId(provinceId);
        if(UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() != 1){
            Long storeResource = UserDetailUtils.getUserDetail().getStoreResourceId();
            if(storeResource != null){
                storeSearchDTO.setStoreResource(storeResource);
            }
        }
        ServiceResponse<Long> countFromES = this.storeApiService.queryStoreCountFromEs(storeSearchDTO);
        return countFromES.getResult().intValue();
    }

    private Integer getStoreNum(Integer enterpriseId, Integer status, String provinceId, String cityId){
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setStatus(status);
        storeSearchDTO.setProvinceId(provinceId);
        storeSearchDTO.setCityId(cityId);
        if(UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() != 1){
            Long storeResource = UserDetailUtils.getUserDetail().getStoreResourceId();
            if(storeResource != null){
                storeSearchDTO.setStoreResource(storeResource);
            }
        }
        ServiceResponse<Long> countFromES = this.storeApiService.queryStoreCountFromEs(storeSearchDTO);
        return countFromES.getResult().intValue();
    }

    private List<Integer> getAuthStore(){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        List<Integer> authStoreIdList = new ArrayList<>();
        if(UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() != 1){
            if (UserDetailUtils.getUserDetail().getStoreResourceInfo().getStoreResourceId() == null) {
                authStoreIdList.add(-1);
            } else {
                ServiceResponse<List<Integer>> authResponse = this.storeWidgetApiService.listStoreInfoIdByStoreWidgetId(enterpriseId, UserDetailUtils.getUserDetail().getStoreResourceInfo().getStoreResourceId());
                if(authResponse.isSuccess()){
                    authStoreIdList = authResponse.getResult();
                }
                if(CollectionUtils.isEmpty(authStoreIdList)){
                    authStoreIdList.add(-1);
                }
            }
        }
        return authStoreIdList;
    }

    private List<MemberCardInfoVO> getMbrArea(Integer userId, Integer enterpriseId){
        AreaSearchDTO areaSearchDTO = new AreaSearchDTO();
        areaSearchDTO.setCuTypes(Arrays.asList(CuTypeEnum.MCU));
        areaSearchDTO.setEnterpriseId(enterpriseId);
        areaSearchDTO.setUserId(userId);
        List<ChannelAreaInfoDTO> listCard = userListFieldApiService.queryCustomAreaInfo(areaSearchDTO).getResult();
        if(CollectionUtils.isNotEmpty(listCard)){
            return listCard.stream().map(dto -> {
                MemberCardInfoVO vo = new MemberCardInfoVO();
                vo.setMbrAreaId(dto.getAreaId().toString());
                vo.setMbrAreaName(dto.getName());
                vo.setOwnType(dto.getOwnerFlag());
                return vo;
            }).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }
}